// Copyright (c) 2019 PaddlePaddle Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include "lite/kernels/arm/range_compute.h"

namespace paddle {
namespace lite {
namespace kernels {
namespace arm {

void RangeCompute::Run() {
  auto& param = Param<operators::RangeParam>();
  // int start = static_cast<int>(param.Start->data<float>()[0]);
  // int end = static_cast<int>(param.End->data<float>()[0]);
  // int step = static_cast<int>(param.Step->data<float>()[0]);
  int start = (param.Start->data<float>()[0]);
  int end = (param.End->data<float>()[0]);
  int step = (param.Step->data<float>()[0]);

  float* out_data = param.Out->mutable_data<float>();
  float value = start;
  for (int i = 0; i < param.Out->dims().production(); ++i) {
    out_data[i] = value;
    value += step;
  }
}

}  // namespace arm
}  // namespace kernels
}  // namespace lite
}  // namespace paddle

REGISTER_LITE_KERNEL(
    range, kARM, kFloat, kNCHW, paddle::lite::kernels::arm::RangeCompute, def)
    .BindInput("Start", {LiteType::GetTensorTy(TARGET(kARM))})
    .BindInput("End", {LiteType::GetTensorTy(TARGET(kARM))})
    .BindInput("Step", {LiteType::GetTensorTy(TARGET(kARM))})
    .BindOutput("Out", {LiteType::GetTensorTy(TARGET(kARM))})
    .Finalize();
